package com.ippon.article.clientmobile.database;

import android.content.Context;
import android.util.Log;

import com.couchbase.lite.BasicAuthenticator;
import com.couchbase.lite.CouchbaseLiteException;
import com.couchbase.lite.DataSource;
import com.couchbase.lite.Database;
import com.couchbase.lite.DatabaseChangeListener;
import com.couchbase.lite.DatabaseConfiguration;
import com.couchbase.lite.Endpoint;
import com.couchbase.lite.Expression;
import com.couchbase.lite.ListenerToken;
import com.couchbase.lite.LogDomain;
import com.couchbase.lite.LogLevel;
import com.couchbase.lite.Query;
import com.couchbase.lite.QueryBuilder;
import com.couchbase.lite.Replicator;
import com.couchbase.lite.ReplicatorConfiguration;
import com.couchbase.lite.Result;
import com.couchbase.lite.ResultSet;
import com.couchbase.lite.SelectResult;
import com.couchbase.lite.URLEndpoint;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

/**
 * Created by jeremie on 04/05/18.
 */

public class DatabaseManagement {

    private final static String TAG = "DatabaseManagement";

    private Database database;
    private ListenerToken databaseListenerToken;
    private Replicator replicator;
    private ListenerToken replicatorListenerToken;

    public void openDatabase(Context androidContext, DatabaseChangeListener listener) throws CouchbaseLiteException {
        // Get the database (and create it if it doesn’t exist).
        DatabaseConfiguration config = new DatabaseConfiguration(androidContext);
        database = new Database("localdb", config);

        databaseListenerToken = database.addChangeListener(listener);
    }

    public void deleteDatabase() throws CouchbaseLiteException {

        // Remove listeners
        replicator.removeChangeListener(replicatorListenerToken);
        database.removeChangeListener(databaseListenerToken);

        // Stop replication
        replicator.stop();
        while (replicator.getStatus().getActivityLevel() != Replicator.ActivityLevel.STOPPED) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                Log.e(TAG, "Replication stop", e);
            }
        }

        database.delete();
    }

    public List<Result> findAllDocuments(String javaClass) throws CouchbaseLiteException {
        // Create a query to fetch documents of type SDK.
        Query query = QueryBuilder.select(SelectResult.property("content"))
                .from(DataSource.database(database))
                .where(Expression.property("javaClass").equalTo(Expression.string(javaClass)));

        ResultSet result = query.execute();
        List<Result> resultsList = result.allResults();
        Log.i(TAG, "Number of rows ::  " + resultsList.size());
        return resultsList;
    }

    public void configureReplication(String gatewayUrl, String gatewayUser, String gatewayPwd) throws URISyntaxException {
        Database.setLogLevel(LogDomain.REPLICATOR, LogLevel.VERBOSE);
        Database.setLogLevel(LogDomain.NETWORK, LogLevel.VERBOSE);

        // Create replicators to push and pull changes to and from the cloud.
        Endpoint targetEndpoint = new URLEndpoint(new URI(gatewayUrl));
        ReplicatorConfiguration replConfig = new ReplicatorConfiguration(database, targetEndpoint);

        replConfig.setReplicatorType(ReplicatorConfiguration.ReplicatorType.PULL);

        // Add authentication.
        replConfig.setAuthenticator(new BasicAuthenticator(gatewayUser, gatewayPwd));
        replConfig.setContinuous(true);

        // Create replicator.
        replicator = new Replicator(replConfig);

        // Listen to replicator change events.
        replicatorListenerToken = replicator.addChangeListener(change -> {
            if (change.getStatus().getError() != null) {
                Log.i(TAG, "Error code ::  " + change.getStatus().getError().getCode());
            }
        });

        // Start replication.
        replicator.start();

    }

}
